/* utf_8b.h: UTF-8b codec */

/*

Copyright (c) 2006, Eric Tiedemann
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:


    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above
      copyright notice, this list of conditions and the following
      disclaimer in the documentation and/or other materials provided
      with the distribution.

    * Neither the name of Eric Tiedemann nor the names of his
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef _E7_UTF8_UTF8_H
#define _E7_UTF8_UTF8_H

#include <stdint.h>
#include <stdio.h>

typedef uint32_t utf_8b_codepoint;
typedef uint8_t utf_8b_byte;

// "merely" the RFC 3629 restricted spec
#define UTF_8B_MAX_CODEPOINT ((utf_8b_codepoint) 0x10FFFF)
// invalid even in maximum ucs-4!
#define UTF_8B_INVALID ((utf_8b_codepoint) 0xFFFFFFFF)

// returns the number of bytes output (up to 4) or 0 on invalid code point
extern int utf_8b_encode(utf_8b_codepoint c, utf_8b_byte *p);
    
struct utf_8b_decoder {
  // codepoint we're accumulating or the final goodie when put() returns 1
  utf_8b_codepoint c;
  // number of continuation 10xxxxxx bytes expected
  // this is also non-zero iff the decodeer is waiting for more 
  // bytes in the middle of a utf-8 sequence
  int expecting;
  // record of the bytes we've seen on the current character attempt.
  // this can be used for error reporting
  utf_8b_byte record[4];
  int recordi;		// index of how much of record we've filled
  
  // this is set on the first byte of multibyte sequences to a mask 
  // that when anded with the second byte must be non-zero.  this 
  // test disallows byte sequences such as 11000001 10100000 (which is
  // properly encoded as 01100000)
  utf_8b_byte zmask;
};

extern void utf_8b_decoder_init(struct utf_8b_decoder *);

extern int utf_8b_decoder_put(struct utf_8b_decoder *,
			      utf_8b_byte b, utf_8b_codepoint *out);

extern void _utf_8b_decoder_bad(struct utf_8b_decoder *);

extern int _utf_8b_decoder_invalid_bytes(struct utf_8b_decoder *,
					utf_8b_codepoint *out);

#endif
